/*
 * This file is part of the Owl Positioning System (OwlPS).
 * OwlPS is a project of the University of Franche-Comté
 * (Université de Franche-Comté), France.
 */


#include "inputlogcsv.hh"
#include "posexcept.hh"
#include "mobile.hh"
#include "calibrationrequest.hh"
#include "referencepoint.hh"

#include <sstream>

using namespace std ;
using std::tr1::unordered_map ;



/* *** Operations *** */


const string InputLogCSV::request_to_csv(const Request &request) const
{
  ostringstream csv_line ;
  if (request.get_mobile() != NULL)
    csv_line << request.get_mobile()->get_mac_addr() ;
  csv_line
      << ';' << static_cast<uint_fast16_t>(request.get_type())
      << ';' << request.get_nb_packets()
      << ';' << request.get_time_sent()
      << ';' ;

  const CalibrationRequest *calibration_request =
    dynamic_cast<const CalibrationRequest*>(&request) ;
  if (calibration_request == NULL)
    csv_line << "0;0;0;0" ;
  else
    {
      csv_line
          << calibration_request->get_reference_point()->get_x() << ';'
          << calibration_request->get_reference_point()->get_y() << ';'
          << calibration_request->get_reference_point()->get_z() << ';'
          << static_cast<int>(calibration_request->get_direction()) ;
    }

  const unordered_map<string, Measurement> &measurements =
    request.get_measurements() ;
  for (unordered_map<string, Measurement>::const_iterator i
       = measurements.begin() ; i != measurements.end() ; ++i)
    csv_line << i->second.to_csv() ;

  csv_line << '\n' ;
  return csv_line.str() ;
}
